package com.zink.scala.fly.kit

import com.zink.scala.fly.FlyPrime
import com.zink.scala.fly.stub.StringCodec._

import java.io.DataInputStream
import java.net.{InetAddress, InetSocketAddress, Socket}
import java.util.concurrent.TimeoutException

object FlyPinger {
  val DEFAULT_SOCKET_TIMEOUT: Int = 50 //ms
  val pingOp: Array[Byte] = Array(0xfa.toByte, 0xb1.toByte, 0x0.toByte, 0x0.toByte)
}

class FlyPinger(socketTimeout: Int = FlyPinger.DEFAULT_SOCKET_TIMEOUT) extends AnyRef with Logging {


  /**
   * Send a ping message to a Fly Space running on the given Address
   * prefer the method that takes an InetAddress. Name to address 
   * conversion can be very slow on some platforms
   * @param host
   * @return String [] of tags or null array for failed ping.
   */   
  def ping(host:String):Option[Array[String]] = ping(InetAddress.getByName(host))

  /**
   *  Send a ping message to a Fly Space running on the given InetAddress and
   *  port.
   * @param addr
   * @return An Option of an array of tags or None for failed ping.
   */
  def ping(addr: InetAddress): Option[Array[String]] = {
    var sock = new Socket()
    sock.setKeepAlive(true)
    
    try {
      sock.bind(null)
      sock.connect(new InetSocketAddress(addr, FlyPrime.FLY_PORT), socketTimeout)
      val dis = new DataInputStream(sock.getInputStream())
      sock.getOutputStream().write(FlyPinger.pingOp)
      // wait for the reply for the specified time.
      val start = System.currentTimeMillis()
      while (dis.available() == 0 && (System.currentTimeMillis() - start) < socketTimeout) Thread.sleep(1)
      
      if (dis.available() == 0 ) throw new TimeoutException("No reply from fly server")
      val tags = (for (i <- 0 until dis.readLong().toInt) yield readString(dis)).toArray
      
      sock.setKeepAlive(false)
      sock.shutdownInput()
      sock.shutdownOutput()
      sock.close()
      Some(tags)
    } catch {
      case e => None
    } finally {
      sock.close()
    }
  }
}
